mfDOWNLOAD <- function(url, archive, t0=NULL, t1=NULL, h0=NULL, h1=NULL, v0=NULL, v1=NULL, natt=NULL, tatt=NULL){
# this function is intended to downoad MODIS HDF files from USGS/LPDAAC's Data Pool.
# A user-defined subset of available scenes can be chosen (tiles and date).
# The file list that is retrieved from the server is matched with an existing MODIS archive in the file system.
#  All files that are not within the archive will be downloaded.

# input: 
# url = data pool url, i.e. the top level of one MODIS product.
#       e.g. "http://e4ftl01.cr.usgs.gov/MOLT/MOD14A2.005/"
# archive = directory where the MODIS archive in the file system is located.
# t0 = start date in format YYYYMMDD
# t1 = end date in format YYYYMMDD
# h0 = start horizontal tile number
# h1 = end horizontal tile number
# v0 = start vertical tile number
# v1 = end vertical tile number
# natt = number of attempts for server connections (default=10)
# tatt = halting time between failed server connections in seconds (default=60)


# details:
# If the archive for a given MODIS product doesn't exist yet, it will be created.
# If no subset is specified (t0/t1, h0/h1, v0/v1), the user is asked to proceed.
#   If yes,  all files will be downloaded (if not already in archive).
# For each server access operation, 10 attempts are made to retrieve html text or hdf files.
#   The function halts for 60s (default) between failed attempts.
#   If the maximum number of attempts is exceeded (defaults to 10), the function will stop with an error.

	if (!file.exists(archive)){
		cat("MODIS archive does not exist; creating archive; ")
		dir.create(archive, recursive=TRUE)
		cat("done\n")}

	if (is.null(t0) & is.null(t1)){
		cat("t0/t1 are not specified. Are you sure to proceed? Download complete time record?\n")
		ans <- ""
		while(!ans %in% c("y", "yes", "n", "no")) {
			cat("yes [y] or no [n]?\n") 
			ans <- scan(file="stdin", what="character", n=1, quiet=TRUE)}
		if (ans == "n") stop("download aborted by user", call.=FALSE)
		if (ans == "y") cat("OK. Proceed with download\n")
	}

	if (is.null(h0) & is.null(h1) & is.null(v0) & is.null(v1)){
		cat("h0/h1 and v0/v1 are not specified. Are you sure to proceed? Download global coverage?\n")
		ans <- ""
		while(!ans %in% c("y", "yes", "n", "no")) {
			cat("yes [y] or no [n]?\n") 
			ans <- scan(file="stdin", what="character", n=1, quiet=TRUE)}
		if (ans == "n") stop("download aborted by user", call.=FALSE)
		if (ans == "y") cat("OK. Proceed with download\n")
	}

	# default values
	if (is.null(t0))   t0 <- 20000101
	if (is.null(t1))   t1 <- 20991231
	if (is.null(h0))   h0 <- 0
	if (is.null(h1))   h1 <- 35
	if (is.null(v0))   v0 <- 0
	if (is.null(v1))   v1 <- 17
	if (is.null(tatt)) tatt <- 60
	if (is.null(natt)) natt <- 10


	# get file list in archive
	arch <- basename(dir(archive, "\\.hdf$", recursive=TRUE))

	# retrieve top level html text
	htm <- ""; class(htm) <- "try-error"; j <- 0
	while(class(htm) == "try-error"){
		if (j != 0) {
			if (j == natt) stop("HTTP server is down. Try again later.", call.=FALSE)
			cat("no response. waiting.", "\n")
			Sys.sleep(tatt)
		}
		htm <- try(readLines(url), silent=TRUE)
		j <- j +1
	}

	# get html lines with links
	pos <- grep("<a href", htm, value=TRUE)
	# extract links
	link <- gsub(".*(<a href[^<]*>).*", "\\1", pos)
	# extract relative url from tag
	url.0 <- gsub(".*\"(.*/).*", "\\1", link)
	# get dates
	dat <- strptime(url.0, "%Y.%m.%d/")
	# remove links that dont contain a date
	rem <- is.na(dat)
	dat <- dat[-rem]
	url.0 <- url.0[-rem]
	# remove links that dont match with user-defined date criteria
	t0 <- strptime(t0, "%Y%m%d")
	t1 <- strptime(t1, "%Y%m%d")
	sub <- dat >= t0 & dat <= t1
	if (sum(sub) == 0){
		cat("no files matching date criteria\n"); flush.console();
		return(1)
	}
	cat(sum(sub), "dates found. start recursice search for matching tiles\n")
	url.1 <- paste(url, url.0[sub], sep="")

	# number of second-level urls
	n <- length(url.1)

	# do for each second-level url, i.e. each date
	for (i in 1:n){

		# retrieve 2nd level html text
		htm <- ""; class(htm) <- "try-error"; j <- 0
		while(class(htm) == "try-error"){
			if (j != 0) {
				if (j == natt) stop("HTTP server is down. Try again later.", call.=FALSE)
				cat("no response. waiting.", "\n")
				Sys.sleep(tatt)
			}
			htm <- try(readLines(url.1[i]), silent=TRUE)
			j <- j + 1
		}

		# get html lines with links
		pos <- grep("<a href", htm, value=TRUE)
		# extract links
		link <- gsub(".*(<a href[^<]*>).*", "\\1", pos)
		# extract file name from tag
		url.0 <- gsub(".*\"(.*)\\\".*", "\\1", link)
		# get only hdf files
		files <- grep("\\.hdf$", url.0, value=TRUE)
		# get full url for each file
		URL <- paste(url.1[i], files, sep="")

		# get files and tiles
		files <- basename(URL)
		tiles <- gsub(".*(h[[:digit:]]{2}v[[:digit:]]{2}).*", "\\1", files)
		h <- as.integer(substr(tiles, 2,3))
		v <- as.integer(substr(tiles, 5,6))

		# remove files that dont match with user-defined tile criteria
		sub <- h >= h0 & h <= h1 & v >= v0 & v <= v1
		files <- files[sub]
		URL <- URL[sub]

		if (length(arch > 0)) URL <- URL[!files %in% arch]
		n1 <- length(URL)

		# determine whether there is a need to proceed
		if (n1 == 0) { cat("archive is already up to date\n"); cat("proceed with recursive searching", n-i, "dates still to be screened.\n"); next }
		cat("downloading", n1, "files", "\n\n")

		# do for each MODIS HDF file
		for (j in 1:n1){

			# try to download file
			dow <- ""; class(dow) <- "try-error"; k <- 0
			while(class(dow) == "try-error"){
				if (k != 0) {
					if (k == natt) stop("HTTP server is down. Try again later.", call.=FALSE)
					cat("no response. waiting.", "\n")
					Sys.sleep(tatt)
				}
				dow <- try(download.file(URL[j], file.path(archive,basename(URL[j])), method="auto", mode='wb', quiet=FALSE, cacheOK=FALSE), silent=TRUE)
				k <- k + 1
			}

		}

		cat("proceed with recursive searching", n-i, "dates still to be screened.\n")

	}


}

